;------------------------------------------------------------------------------------------------
;  NetFoss Utilities DOOR32.SYS builder
;  Version 0.01
;  Written by Mike Ehlert - PC Micro Systems, Inc. 
;  http://pcmicro.com/netfoss
;  Released as Public Domain, Licenced Open Source Code.
;  Read Net2BBS.txt for license details.
; 
;  Compile in MASM32, using Qeditor > Project > "Console Assemble and Link"
;  or link using Polink.exe for optimum file size.
;------------------------------------------------------------------------------------------------

; This utility currently only has one function: Read a DOOR.SYS and build a DOOR32.SYS using
; the Socket Handle found in the Environment Variable %SOCKET%.
; You can optionally pass a command line parameter with the path to the node's drop files.

.386
.model flat,stdcall
option casemap:none

include \masm32\include\windows.inc
include \masm32\include\masm32.inc
include \masm32\include\kernel32.inc

includelib \masm32\lib\kernel32.lib
includelib \masm32\lib\masm32.lib

WriteBuffer  proto: DWORD
WriteLine    proto :DWORD,:DWORD
FindLine     proto :DWORD,:DWORD
ReadMyFile   proto :DWORD    

.data
rpos       dd  0 ; current relative postion in file
curline    dd  1 ; current DOOR.SYS line being read
findline   dd  0 ; line number to find

; Hardcoded Door32.sys lines:
line1  db "2",13,10,0         ; Socket Mode
line4  db "NetFoss",13,10,0   ; BBS Name can go here
line10 db "1",13,10,0         ; Emulation 1=ANSI

TitleText         db 13,10,"NetFoss Utilities DOOR32.SYS builder version 0.01",13,10,0
DoorSysFileName   db "DOOR.SYS",0
Door32SysFileName db "DOOR32.SYS",0
FileNotFound      db " file not found!",13,10,0
CreateFileFailed  db "Failed to create DOOR32.SYS",13,10,0
NoEnvironmentVar  db "Environment Variable %SOCKET% not found!",13,10,0
EnvVar            db "SOCKET",0
CRLF              db 13,10,0
CreatedTxt        db " created.",13,10,0
Slash db "\",0
Bell  db 7,0

.data?
hDoor32File   dd ?
written       dd ?
linebuffer    db  256 dup(?)
CommandLine1  db  128 dup(?)
DoorSysPath   db  128 dup(?)
Door32SysPath db  128 dup(?)
EnvVarbuf     db    8 dup(?)


;----- door32.sys drop file format -----------------------------------------------------------------
;
; 2                            Line 1 : Comm type (0=local, 1=serial, 2=telnet)
; 0                            Line 2 : Comm or socket handle
; 38400                        Line 3 : Baud rate
; Mystic 1.07                  Line 4 : BBSID (software name and version)
; 1                            Line 5 : User record position (1-based)
; James Coyle                  Line 6 : User's real name
; g00r00                       Line 7 : User's handle/alias
; 255                          Line 8 : User's security level
; 58                           Line 9 : User's time left (in minutes)
; 1                            Line 10: Emulation (0 =Ascii 1 =Ansi 2 =Avatar 3 =RIP 4 =Max Graphics)
; 1                            Line 11: Current node number
;----------------------------------------------------------------------------------------------------

.code

start:

 	invoke StdOut, ADDR TitleText   ; Display title line

	;mov byte ptr DoorSysPath, 0     ; Zero (terminate) the string
	;mov byte ptr Door32SysPath, 0   ; Zero (terminate) the string

	invoke ArgClC,1,ADDR CommandLine1
	test eax,eax ; check for command line
	jz NoCmdParms ; no command line, it must be an application

	;eax=number of parameters
	invoke lstrlen,ADDR CommandLine1 ; Get length of CommandLine 1 Parameter
	mov ebx,eax
	dec ebx
	cmp byte ptr CommandLine1[ebx],"\" ; Make sure the path includes a trailing backslash
	je havetrailslash
	inc ebx
	mov word ptr CommandLine1[ebx],05C00h ; Append Slash, 0
	;invoke lstrcat, ADDR CommandLine1, ADDR Slash	; Alternative method to append Slash


havetrailslash:

	invoke lstrcpy, ADDR DoorSysPath, ADDR CommandLine1         ; Copy CommandLine1 to the DoorSysPath
	invoke lstrcpy, ADDR Door32SysPath, ADDR CommandLine1       ; Copy CommandLine1 to the Door32SysPath

NoCmdParms:

	invoke lstrcat, ADDR DoorSysPath, ADDR DoorSysFileName	; Append the Door.sys   filename to the end of the path
	invoke lstrcat, ADDR Door32SysPath, ADDR Door32SysFileName	; Append the Door32.sys filename to the end of the path


	invoke GetEnvironmentVariable,addr EnvVar,addr EnvVarbuf, 6 ; Get the SOCKET environment variable (6 chars max).
	.IF eax==0
		invoke StdOut, Addr NoEnvironmentVar ; No Environment Variable was found
		jmp Error_Exit
	.ENDIF

	invoke lstrlen,ADDR EnvVarbuf ; Get length of Environment Variable
	mov ebx,eax
	mov dword ptr EnvVarbuf[ebx],0A0Dh  ; Append CR/LF to the end of the line


; Read DOOR.SYS file into memory
	invoke ReadMyFile, ADDR DoorSysPath ; Open and read the DOOR.SYS file
	jc nodoorsysfile		; get out if the file was not found
	push esi			; save the file pointer so we can later Globalfree it


; Create DOOR32.SYS file
	invoke DeleteFile,ADDR Door32SysPath  ; attempt to delete the old one first
	invoke CreateFile,ADDR Door32SysPath,GENERIC_WRITE,FILE_SHARE_READ,NULL,OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL
        .IF eax==INVALID_HANDLE_VALUE
	    invoke StdOut, Addr CreateFileFailed; get out if the file was not found
	    jmp Error_Exit
        .endif
	mov hDoor32File,eax

; Here is where the magic happens:

	invoke WriteBuffer,  ADDR line1        ; write line1  (Comm Type)      - hard coded as "2"
	invoke WriteBuffer,  ADDR EnvVarbuf    ; write line2  (Socket Handle)  - from environment variable
	invoke WriteLine,2,  ADDR linebuffer   ; write line3  (Baud Rate)      - from door.sys line 2
	invoke WriteBuffer,  ADDR line4        ; write line4  (BBS Software)   - hard coded as "NetFoss"
	invoke WriteLine,26, ADDR linebuffer   ; write line5  (User Record)    - from door.sys line 26
	invoke WriteLine,10, ADDR linebuffer   ; write line6  (User Name)      - from door.sys line 10
	invoke WriteLine,36, ADDR linebuffer   ; write line7  (User Handle)    - from door.sys line 36
	invoke WriteLine,15, ADDR linebuffer   ; write line8  (Security Level) - from door.sys line 15
	invoke WriteLine,19, ADDR linebuffer   ; write line9  (Minutes left)   - from door.sys line 19
	invoke WriteBuffer,  ADDR line10       ; write line10 (Emulation)      - hard coded for ANSI. We could analyze door.sys line 20
	invoke WriteLine,4,  ADDR linebuffer   ; write line11 (Node Number)    - from door.sys line 4

	invoke CloseHandle,hDoor32File ; Close DOOR32.SYS

	pop eax
	invoke GlobalFree,eax 	; free the Door.sys file allocated memory

	invoke StdOut, Addr Door32SysPath ; Display the full DOOR32.SYS Path/Filename
	invoke StdOut, Addr CreatedTxt    ; and a "Created" message.
	invoke  ExitProcess,0             ; All Done


nodoorsysfile:
	invoke StdOut, Addr DoorSysPath		; Display the DOOR.SYS Path
	invoke StdOut, Addr FileNotFound		; and a "File not found" message.


Error_Exit:
	invoke StdOut, Addr Bell           ; make some noise
	invoke Sleep,1000
	invoke StdOut, Addr Bell           ; make more noise
	invoke Sleep,2000
	invoke  ExitProcess,1              ; Exit with ErrorLevel 1


;--------------------------------------------------------------------------------------------
WriteBuffer proc l_buff:dword

; Write an existing line of text to the next line of the door32.sys file

       invoke lstrlen,l_buff	   ; Get size of the string
       mov ebx,eax
	invoke WriteFile,hDoor32File,l_buff,ebx,ADDR written,0

       test eax,eax              ; check for errors
       clc                       ; clear carry to assume sucess
       jnz WB_Exit                ; non-zero result from WriteFile indicates success
       stc                       ; set carry if failure
WB_Exit:
	ret

WriteBuffer endp

;--------------------------------------------------------------------------------------------
WriteLine proc f_line:DWORD,l_buff:dword

; Write a specified line of text taken from door.sys to the next line of the door32.sys file

	invoke FindLine,f_line,l_buff ; find the specified line number
	jc WL_Exit                    ; check for errors
	invoke lstrlen,l_buff         ; Get the length of characters on this line
        mov ebx,eax
        mov edi,l_buff
        mov word ptr [edi+ebx],0A0Dh  ; Append CR/LF to the end of the line
        inc ebx                       ; Adjust length of line to write
        inc ebx
	invoke WriteFile,hDoor32File,l_buff,ebx,ADDR written,0
        test eax,eax                  ; check for errors
        clc                           ; clear carry to assume sucess
        jnz WL_Exit             ; non-zero result from WriteFile indicates success
        stc ; set carry if failure
WL_Exit:
	ret

WriteLine endp

;--------------------------------------------------------------------------------------------
FindLine proc f_line:DWORD,l_buff:dword

; Finds a line number in a text file located at ESI, and copies it to the line buffer

	mov rpos,0
	mov curline,0
	findline_loop:
	invoke  readline,esi,l_buff,rpos
	test    eax,eax
	jz      endoffile
	mov     rpos,eax
	inc curline
	mov eax,curline
	cmp eax,f_line
	jb  findline_loop
	clc               ; indicate line found
	ret

endoffile:
	stc ; indicate line not found
	ret

FindLine endp


;------------------------------------------------------------------------------------------------
ReadMyFile proc near lpfilename

; This opens a file (if the file already exists), and reads the entire file into memory.
; Exit with Carry Set to indicate an error
; Exit with ESI= Global Memory handle (to be released) if successful
; AND ECX= bytes read

local hTempFile
local bytesread

	invoke GetFileAttributes, lpfilename		   ; Check if the file exists
	inc eax
	jz readmyfile_error

	invoke CreateFile,lpfilename,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_ARCHIVE,NULL
	mov hTempFile,eax
	invoke GetFileSize,eax,NULL	; get the file size
	push eax
	invoke GlobalAlloc,GMEM_FIXED,eax	;  allocate at least that much memory (Destroys ECX and ?)
	pop ecx
	mov esi, eax				;  Save the memory Handle.
	invoke	ReadFile,hTempFile,	;  Read the File
			esi,				;  Buffer Address
			ecx,				;  Max Amount to send
			ADDR bytesread, 		;  In: bytes actually read
			NULL				;  overlapped

	or eax,eax
	jz readmyfile_error ; This may be overkill since the file exists

		
	invoke CloseHandle, hTempFile		   ; close the file handle
	mov ecx, bytesread			   ; Return bytes read in ECX
	clc
	ret

readmyfile_error:
	;print "file not found",10,13
;pop esi
	stc
	ret

ReadMyFile endp

;------------------------------------------------------------------------------------------------


END start

